import { useEffect, useState } from "preact/hooks";

export const useRemoteStream = <T = string>(url: string, transformer?: (e: string) => T) : [T[], boolean, string | null] => {
  const [items, setItems] = useState<T[]>([]);
  const [isLoading, setIsLoading] = useState(true);
  const [error, setError] = useState(null);

  useEffect(() => {
    const abortController = new AbortController();

    async function fetchStream() {
      try {
        const response = await fetch(url, {
          signal: abortController.signal
        });
        
        if (!response.ok) throw new Error(`HTTP error! status: ${response.status}`);
        if (!response.body) throw new Error('ReadableStream not supported');

        const reader = response.body.getReader();
        const decoder = new TextDecoder();
        
        while (true) {
          const { done, value } = await reader.read();
          if (done) break;
          
          const chunk = decoder.decode(value);
          const batchItems = chunk.trim().split('\n')
            .filter(line => line.length > 0)
            .map(line => transformer?.(line) ?? JSON.parse(line))
            .flat();
            
          setItems(prev => [...prev, ...batchItems]);
        }
      } catch (err:any) {
        if (err?.name === 'AbortError') return;
        setError(err.message);
      } finally {
        setIsLoading(false);
      }
    }

    fetchStream();
    return () => abortController.abort();
  }, [])

  return [items, isLoading, error];
}

